

#include "elx.preface.h"
#include "elx.begin.h"
#include "elx.error.h"
#include "elx.point.h"
#include "elx.buffer.h"
#include "elx.heap.h"
#include "elx.article.h"
#include "elx.record.h"
#include "elx.archive.h"
#include "elx.kernel.h"
#include "elx.state.h"


const OLECHAR * error::FunctionTable [] =
{
	L"elxOpenBuffer",
	L"elxCheckBuffer",
	L"elxCreateBuffer",
	L"elxFindBuffer",
	L"elxWriteBuffer",
	L"elxReadBufferNext",
	L"elxReadBufferSync",
	L"elxBarLag",

	L"elxOpenRecord",
	L"elxWriteRecord",
	L"elxReadRecord",
	L"elxSaveRecord"

};


const OLECHAR * error::FaultTable [] =
{
	L"Void Context",
	L"Reserved Name",
	L"Invalid Specification",
	L"Null Match",
	L"Incompatible Type",
	L"Invalid Register",
	L"Invalid Handle",
	L"Reserved File"
	L"Limit Violation"
};


void error::Log
( IEasyLanguageObject * ELObjP, integer Kind, integer Location )
{
	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 && StateP->EnableTraps() )
	{
		IEasyLanguageErrors * ELErrP = ELObjP->Errors;

		TSRuntimeErrorItem Item;
		Item.sCompany = SysAllocString( L"TradeStation Securities" );
		Item.sErrorLocation = SysAllocString( L"ELX Library" );
		Item.sErrorCategory = SysAllocString( L"ELX Fault" );
		Item.sShortString = SysAllocString( L"ELX Fault" );
		Item.sSourceString = SysAllocString( L"\"%1\" in %2" );
		Item.sLongString = NULL;
		Item.nParameters = 2;
		Item.nErrorCode = 0;
		Item.nErrorID = 0;
    
		int EH = ELErrP->RegisterError( &Item );
    
		SysFreeString( Item.sSourceString );
		SysFreeString( Item.sShortString );
		SysFreeString( Item.sErrorCategory );
		SysFreeString( Item.sErrorLocation );
		SysFreeString( Item.sCompany );
    
		SAFEARRAY * ArrayP = SafeArrayCreateVector( VT_VARIANT, 0, 2 );
    
		{
			long EI = 0; 
			VARIANT SubText;
			SubText.vt = VT_BSTR;
			SubText.bstrVal = SysAllocString( error::FaultTable[ Kind ] );
			SafeArrayPutElement( ArrayP, &EI, ( void * )&SubText );
			SysFreeString( SubText.bstrVal );
		}
    
		{
			long EI = 1; 
			VARIANT SubText;
			SubText.vt = VT_BSTR;
			SubText.bstrVal = SysAllocString( error::FunctionTable[ Location ] );
			SafeArrayPutElement( ArrayP, &EI, ( void * )&SubText );
			SysFreeString( SubText.bstrVal );
		}
    
    
		VARIANT ArgList;
		ArgList.vt = VT_ARRAY | VT_VARIANT;
		ArgList.parray = ArrayP;
    
		ELErrP->RaiseRuntimeErrorEx( EH, &ArgList );
	}
}




#include "elx.end.h"

