

#include "elx.preface.h"
#include "elx.begin.h"
#include "elx.error.h"
#include "elx.point.h"
#include "elx.buffer.h"
#include "elx.heap.h"
#include "elx.article.h"
#include "elx.record.h"
#include "elx.archive.h"
#include "elx.kernel.h"
#include "elx.state.h"
#include "elx.end.h"

using namespace elx;

literal
{
	xSecondsPerMinute = 60,
	xSecondsPerDay = 86400,
	xSecondsPerWeek = 604800,
	xMinutesPerDay = 1440,
	xExtraneousDays = 657000  // 1800 years
};


integer xReadBuffer
( IEasyLanguageObject * ELObjP, int BH, char * DataRegP, char * LagRegP, integer Mode, integer Location );


integer xTypeOf	 
( const character * TypeP );


integer xTypeOf 
( IEasyLanguageVariable * ELVarP );


real xElapsedSeconds
( real Instant );



dll_entry( int ) elxOpenBuffer
( IEasyLanguageObject * ELObjP, char * NameP, char * TypeP, int Scale, bool Flush )
{
	integer BH = 0;
	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		integer NameLen = xstrlen( NameP );
		integer Type = xTypeOf( TypeP );
		if ( NameLen > 0 && Type != _void && Scale > 0 && Scale < heap::MaxScale )
		{
			text Name ( NameP );
			BH = StateP->FindBuffer( Name );
			if ( BH == 0 )
			{
				BH = StateP->AddBuffer( text( NameP ), Type, Scale );
			}
			else
			{
				integer N; integer H; heap * HeapP = StateP->Heap();
				for ( N = Scale, H = BH; N > 0 && HeapP->TestBuffer( H ) && HeapP->TypeBuffer( H ) == Type; --N, ++H );
				if ( N > 0 )
				{
					BH = 0;
					error::Log( ELObjP, error::IncompatibleType, error::OpenBuffer );
				}
				else if ( Flush )
				{
					for ( integer BI = 0; BI < Scale; ++BI ) HeapP->ResetBuffer( BH + BI );
				}
			}
		}
		else
		{
			error::Log( ELObjP,  error::InvalidSpec, error::OpenBuffer );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::OpenBuffer );
	}

	return BH;
}


dll_entry( int ) elxCheckBuffer
( IEasyLanguageObject * ELObjP, char * NameP )
{
	integer Indicator = 0;
	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP == 0 )
	{
		error::Log( ELObjP,  error::VoidContext, error::CheckBuffer );
	}
	else if ( xstrlen( NameP ) > 0 ) 
	{
		Indicator = StateP->Heap()->FindBuffer( text( NameP ) );
	}
	return Indicator;
}



dll_entry( int ) elxCreateBuffer    // OBSOLETE!
( IEasyLanguageObject * ELObjP, char * NameP, char * TypeP, int Scale )
{
	integer BH = 0;
	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		integer NameLen = xstrlen( NameP );
		integer Type = xTypeOf( TypeP );
		if ( NameLen > 0 && Type != _void && Scale > 0 && Scale < heap::MaxScale )
		{
			BH = StateP->AddBuffer( text( NameP ), Type, Scale );
			if ( BH == 0 )
			{
				error::Log( ELObjP, error::ReservedName, error::CreateBuffer );
			}
		}
		else
		{
			error::Log( ELObjP,  error::InvalidSpec, error::CreateBuffer );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::CreateBuffer );
	}

	return BH;
}


dll_entry( int ) elxFindBuffer    // OBSOLETE!
( IEasyLanguageObject * ELObjP, char * NameP )
{
	integer BH = 0;
	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		if ( xstrlen( NameP ) > 0 ) BH = StateP->FindBuffer( text( NameP ) );
		if ( BH == 0 )
		{
			error::Log( ELObjP, error::NullMatch, error::FindBuffer );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::FindBuffer );
	}
	return BH;
}



dll_entry( int ) elxWriteBuffer
( IEasyLanguageObject * ELObjP, int BH, char * DataRegP )
{
	int Output = 0;
	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		heap * HeapP = StateP->Heap();
		if ( HeapP->TestBuffer( BH ) )
		{
			IEasyLanguageVariable * ELDataVarP = ELObjP->Variables[ DataRegP ];
			if ( ELDataVarP != 0 )
			{
				integer Type = HeapP->TypeBuffer( BH );
				if ( xTypeOf( ELDataVarP ) == Type )
				{
					real Instant = ELObjP->DateTimeMD[ dataDefault ]->AsDateTime[ 0 ]; 

					switch( Type )
					{
						case _real:
						{
							real Value = ELDataVarP->AsDouble[ 0 ];
							HeapP->WriteBuffer( BH, Instant, Value );
						}
						break;
             
						case _integer:
						{
							integer Value = ELDataVarP->GetAsInteger( 0 );   // missing property def
							HeapP->WriteBuffer( BH, Instant, Value );
						}
						break;
             
						case _condition:
						{
							condition Value = ( ELDataVarP->AsTrueFalse[ 0 ] != 0 );
							HeapP->WriteBuffer( BH, Instant, Value );
						}
						break;
             
						case _text:
						{
							text Value ( plain );
							Value.Append( ( character * )ELDataVarP->AsString[ 0 ] );
							HeapP->WriteBuffer( BH, Instant, Value );
						}
						break;
					}

					Output = 1;
				}
				else
				{
					error::Log( ELObjP, error::IncompatibleType, error::WriteBuffer );
				}
			}
			else
			{
				error::Log( ELObjP,  error::InvalidRegister, error::WriteBuffer );
			}
		}
		else
		{
			error::Log( ELObjP,  error::InvalidHandle, error::WriteBuffer );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::WriteBuffer );
	}
	return Output;
}


dll_entry( int ) elxReadBufferNext
( IEasyLanguageObject * ELObjP, int BH, char * DataRegP, char * LagRegP )
{
	return xReadBuffer( ELObjP, BH, DataRegP, LagRegP, buffer_::ReadNext, error::ReadBufferNext );
}


dll_entry( int ) elxReadBufferSync
( IEasyLanguageObject * ELObjP, int BH, char * DataRegP, char * LagRegP )
{
	return xReadBuffer( ELObjP, BH, DataRegP, LagRegP, buffer_::ReadSync, error::ReadBufferSync ); 
}


integer xReadBuffer
( IEasyLanguageObject * ELObjP, int BH, char * DataRegP, char * LagRegP, integer Mode, integer Location )
{
	integer Input = 0;

	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		heap * HeapP = StateP->Heap();
		if ( HeapP->TestBuffer( BH ) )
		{
			IEasyLanguageVariable * ELDataVarP = ELObjP->Variables[ DataRegP ];
			IEasyLanguageVariable * ELLagVarP = ELObjP->Variables[ LagRegP ];
			if ( ELDataVarP != 0 && ELLagVarP != 0 && xTypeOf( ELLagVarP ) == _real )
			{
		 		integer Type = HeapP->TypeBuffer( BH );
				if ( xTypeOf( ELDataVarP ) == Type )
				{
					condition Indicator; real Instant;
					real KeyInstant = ELObjP->DateTimeMD[ dataDefault ]->AsDateTime[ 0 ]; 

					switch( Type )
					{
						case _real:
						{
							real Value; Indicator = HeapP->ReadBuffer( BH, Mode, KeyInstant, Value, Instant );
							if ( Indicator ) ELDataVarP->AsDouble[ 0 ] = Value;
						}
						break;
		
						case _integer:
						{
							integer Value; Indicator = HeapP->ReadBuffer( BH, Mode, KeyInstant, Value, Instant );
							if ( Indicator ) ELDataVarP->PutAsInteger( 0, Value );  // missing property def
						}
						break;

						case _condition:
						{
							condition Value; Indicator = HeapP->ReadBuffer( BH, Mode, KeyInstant, Value, Instant );
							if ( Indicator ) ELDataVarP->AsTrueFalse[ 0 ] = Value;
						}
						break;
	
						case _text:
						{
							text Value; Indicator = HeapP->ReadBuffer( BH, Mode, KeyInstant, Value, Instant );
							if ( Indicator ) ELDataVarP->AsString[ 0 ] = _bstr_t ( ( const character * )Value );
						}
						break;

					}
					if ( Indicator ) 
					{
						real Lag = ( KeyInstant == Instant ) ? 0 : xElapsedSeconds( KeyInstant ) - xElapsedSeconds( Instant );
						ELLagVarP->AsDouble[ 0 ] = Lag;
						Input = 1;
					}
				}
				else
				{
					error::Log( ELObjP, error::IncompatibleType, Location );
				}
			}
			else
			{
				error::Log( ELObjP, error::InvalidRegister, Location );
			}
		}
		else
		{
			error::Log( ELObjP, error::InvalidHandle, Location );
		}
	}
	else
	{
		error::Log( ELObjP, error::VoidContext, Location );
	}

	return Input;
}



dll_entry( int ) elxBarLag
( IEasyLanguageObject * ELObjP, double TimeLag, int BarType, int BarInterval )
{
	// TimeLag is measured in seconds

	integer BarLag = 0;

	if
	(
	  ( TimeLag >= 0 ) &&
	  ( BarType >= 1 && BarType <= 3 ) &&
	  ( BarType != 1 || ( BarInterval >= 1 && BarInterval <= xMinutesPerDay ) )
	)
	{
		real N;
		if ( BarType == 1 )  // minute
		{
			N = TimeLag / ( BarInterval * xSecondsPerMinute ); 
		}
		else if ( BarType == 2 )  // daily
		{
			N = TimeLag / xSecondsPerDay;
		}
		else // ( BarType == 3 )  // weekly
		{
			N = TimeLag / xSecondsPerWeek; 
		}
		element::RoundUp( N );
		BarLag = ( integer )N;
	}
	else
	{
		error::Log( ELObjP, error::InvalidSpec, error::BarLag );
	}

	return BarLag;
}


real xElapsedSeconds
( real Instant )
{
	confirm( Instant > 0 );
	Instant = ( Instant - xExtraneousDays ) * xSecondsPerDay;
	if ( !element::RoundNear( Instant ) ) element::RoundDown( Instant );
	return Instant;
}


integer xTypeOf		// EL type id from type token
( const character * TypeP )
{
	integer Type;
	if ( xstricmp( TypeP, $"double" ) == 0 || xstricmp( TypeP, $"float" ) == 0 ||  xstricmp( TypeP, $"numeric" ) == 0 )
	{
		Type = _real;
	}
	else if ( xstricmp( TypeP, $"int" ) == 0 || xstricmp( TypeP, $"integer" ) == 0 )
	{
		Type = _integer;
	}
	else if ( xstricmp( TypeP, $"string" ) == 0 || xstricmp( TypeP, $"text" ) == 0 )
	{
		Type = _text;
	}
	else if ( xstricmp( TypeP, $"truefalse" ) == 0 || xstricmp( TypeP, $"bool" ) == 0 || xstricmp( TypeP, $"boolean" ) == 0 )
	{
		Type = _condition;
	}
	else
	{
		Type = _void;
	}
	return Type;
}


integer xTypeOf 
( IEasyLanguageVariable * ELVarP )
{
	integer Type;
	switch ( ( integer )ELVarP->GetDataType() )
	{
		case dtDouble:
		{
			Type = _real;
		}
		break;

		case dtInteger:
		{
			Type = _integer;
		}
		break;

		case dtTrueFalse:
		{
			Type = _condition;
		}
		break;

		case dtString:
		{
			Type = _text;
		}
		break;

		default:
		{
			Type = _void;
		}
		break;

	}
	return Type;
}


