

#include "elx.preface.h"
#include "elx.begin.h"
#include "elx.error.h"
#include "elx.point.h"
#include "elx.buffer.h"
#include "elx.heap.h"
#include "elx.article.h"
#include "elx.record.h"
#include "elx.archive.h"
#include "elx.kernel.h"
#include "elx.state.h"
#include "elx.end.h"

using namespace elx;


dll_entry( int ) elxOpenRecord
( IEasyLanguageObject * ELObjP, char * NameP )
{
	integer RH = 0;

	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		integer Reason; RH = StateP->OpenRecord( text( NameP ), Reason );
		if ( RH == 0 )
		{
			Reason = ( Reason == record::InvalidName ) ? error::InvalidSpec : error::LimitViolation;	
			error::Log( ELObjP, Reason, error::OpenRecord );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::OpenRecord );
	}

	return RH;
}


dll_entry( int ) elxReadRecord
( IEasyLanguageObject * ELObjP, int RH, char * KeyP, char * RegP )
{
	integer N = 0;

	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		archive * ArchiveP = StateP->Archive();
		if ( ArchiveP->TestRecord( RH ) )
		{
			IEasyLanguageVariable * ELVarP = ELObjP->Variables[ RegP ];
			if ( ELVarP != 0 )
			{
				text Key ( KeyP );
				condition Indicator = false; 
				switch( ( integer )ELVarP->DataType )
				{
					case dtInteger:
					{
						integer Value; Indicator = ArchiveP->ReadRecord( RH, Key, Value );
						if ( Indicator ) ELVarP->PutAsInteger( 0, Value );  // missing property def
					}
					break;

					case dtDouble:
					{
						real Value; Indicator = ArchiveP->ReadRecord( RH, Key, Value );
						if ( Indicator ) ELVarP->AsDouble[ 0 ] = Value;
					}
					break;

					case dtString:
					{
						text Value; Indicator = ArchiveP->ReadRecord( RH, Key, Value );
						if ( Indicator ) ELVarP->AsString[ 0 ] = _bstr_t ( ( const character * )Value );
					}
					break;

					case dtTrueFalse:
					{
						condition Value; Indicator = ArchiveP->ReadRecord( RH, Key, Value );
						if ( Indicator ) ELVarP->AsTrueFalse[ 0 ] = Value;
					}
					break;

					default:
					{
						confirm( false );
					}
					break;
				}
				if ( Indicator ) N = 1;
			}
			else
			{
				error::Log( ELObjP,  error::InvalidRegister, error::ReadRecord );
			}
		}
		else
		{
			error::Log( ELObjP, error::InvalidHandle, error::ReadRecord );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::ReadRecord );
	}

	return N;
}


dll_entry( int ) elxWriteRecord
( IEasyLanguageObject * ELObjP, int RH, char * KeyP, char * RegP )
{
	integer N = 0;

	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		archive * ArchiveP = StateP->Archive();
		if ( ArchiveP->TestRecord( RH ) )
		{
			IEasyLanguageVariable * ELVarP = ELObjP->Variables[ RegP ];
			if ( ELVarP != 0 )
			{
				text Key ( KeyP );
				switch( ( integer )ELVarP->DataType )
				{
					case dtInteger:
					{
						integer Value = ELVarP->GetAsInteger( 0 );  // missing property def
						ArchiveP->WriteRecord( RH, Key, Value );
					}
					break;

					case dtDouble:
					{
						real Value = ELVarP->AsDouble[ 0 ];
						ArchiveP->WriteRecord( RH, Key, Value );
					}
					break;

					case dtString:
					{
						text Value ( plain );
						Value.Append( ( character * )ELVarP->AsString[ 0 ] );
						ArchiveP->WriteRecord( RH, Key, Value );
					}
					break;

					case dtTrueFalse:
					{
						condition Value = ( ELVarP->AsTrueFalse[ 0 ] != 0 );
						ArchiveP->WriteRecord( RH, Key, Value );
					}
					break;

					default:
					{
						confirm( false );
					}
					break;

				}
				N = 1;
			}
			else
			{
				error::Log( ELObjP,  error::InvalidRegister, error::WriteRecord );
			}
		}
		else
		{
			error::Log( ELObjP, error::InvalidHandle, error::WriteRecord );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::WriteRecord );
	}

	return N;
}



dll_entry( int ) elxSaveRecord
( IEasyLanguageObject * ELObjP, int RH )
{
	integer N = 0;
	state * StateP = ( state * )ELObjP->Tag;
	if ( StateP != 0 )
	{
		archive * ArchiveP = StateP->Archive();
		if ( ArchiveP->TestRecord( RH ) )
		{
			integer Reason; N = ArchiveP->SaveRecord( RH, Reason ); 
			if ( N == 0 && Reason == record::NoAccess ) error::Log( ELObjP, error::ReservedFile, error::SaveRecord );
		}
		else
		{
			error::Log( ELObjP, error::InvalidHandle, error::SaveRecord );
		}
	}
	else
	{
		error::Log( ELObjP,  error::VoidContext, error::SaveRecord );
	}
	return N;
}

